#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_config
author: Peter Sprygada (@privateip)
short_description: Module to manage configuration sections.
description:
  - Cisco IOS configurations use a simple block indent file syntax for segmenting configuration
    into sections.  This module provides an implementation for working with IOS configuration
    sections in a deterministic way.
version_added: 1.0.0
extends_documentation_fragment:
  - cisco.ios.ios
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - Abbreviated commands are NOT idempotent, see
    U(https://docs.ansible.com/ansible/latest/network/user_guide/faq.html#why-do-the-config-modules-always-return-changed-true-with-abbreviated-commands)
  - To ensure idempotency and correct diff the configuration lines in the relevant module options should be similar to how they
    appear if present in the running configuration on device including the indentation.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  lines:
    description:
      - The ordered set of commands that should be configured in the section. The commands
        must be the exact same commands as found in the device running-config to ensure
        idempotency and correct diff. Be sure to note the configuration command syntax as
        some commands are automatically modified by the device config parser.
    type: list
    elements: str
    aliases:
      - commands
  parents:
    description:
      - The ordered set of parents that uniquely identify the section or hierarchy the
        commands should be checked against.  If the parents argument is omitted, the
        commands are checked against the set of top level or global commands.
    type: list
    elements: str
  src:
    description:
      - Specifies the source path to the file that contains the configuration or configuration
        template to load.  The path to the source file can either be the full path on
        the Ansible control host or a relative path from the playbook or role root directory. This
        argument is mutually exclusive with I(lines), I(parents). The configuration lines in the
        source file should be similar to how it will appear if present in the running-configuration
        of the device including the indentation to ensure idempotency and correct diff.
    type: str
  before:
    description:
      - The ordered set of commands to push on to the command stack if a change needs
        to be made.  This allows the playbook designer the opportunity to perform configuration
        commands prior to pushing any changes without affecting how the set of commands
        are matched against the system.
    type: list
    elements: str
  after:
    description:
      - The ordered set of commands to append to the end of the command stack if a change
        needs to be made.  Just like with I(before) this allows the playbook designer
        to append a set of commands to be executed after the command set.
    type: list
    elements: str
  match:
    description:
      - Instructs the module on the way to perform the matching of the set of commands
        against the current device config.  If match is set to I(line), commands are
        matched line by line.  If match is set to I(strict), command lines are matched
        with respect to position.  If match is set to I(exact), command lines must be
        an equal match.  Finally, if match is set to I(none), the module will not attempt
        to compare the source configuration with the running configuration on the remote
        device.
    choices:
      - line
      - strict
      - exact
      - none
    type: str
    default: line
  replace:
    description:
      - Instructs the module on the way to perform the configuration on the device.
        If the replace argument is set to I(line) then the modified lines are pushed
        to the device in configuration mode.  If the replace argument is set to I(block)
        then the entire command block is pushed to the device in configuration mode
        if any line is not correct.
    default: line
    choices:
      - line
      - block
    type: str
  multiline_delimiter:
    description:
      - This argument is used when pushing a multiline configuration element to the
        IOS device.  It specifies the character to use as the delimiting character.  This
        only applies to the configuration action.
    default: "@"
    type: str
  backup:
    description:
      - This argument will cause the module to create a full backup of the current C(running-config)
        from the remote device before any changes are made. If the C(backup_options)
        value is not given, the backup file is written to the C(backup) folder in the
        playbook root directory or role root directory, if playbook is part of an ansible
        role. If the directory does not exist, it is created.
    type: bool
    default: false
  running_config:
    description:
      - The module, by default, will connect to the remote device and retrieve the current
        running-config to use as a base for comparing against the contents of source.
        There are times when it is not desirable to have the task get the current running-config
        for every task in a playbook.  The I(running_config) argument allows the implementer
        to pass in the configuration to use as the base config for comparison.
        The configuration lines for this option should be similar to how it will appear if present
        in the running-configuration of the device including the indentation to ensure idempotency
        and correct diff.
    type: str
    aliases:
      - config
  defaults:
    description:
      - This argument specifies whether or not to collect all defaults when getting
        the remote device running config.  When enabled, the module will get the current
        config by issuing the command C(show running-config all).
    type: bool
    default: false
  save_when:
    description:
      - When changes are made to the device running-configuration, the changes are not
        copied to non-volatile storage by default.  Using this argument will change
        that before.  If the argument is set to I(always), then the running-config will
        always be copied to the startup-config and the I(modified) flag will always
        be set to True.  If the argument is set to I(modified), then the running-config
        will only be copied to the startup-config if it has changed since the last save
        to startup-config.  If the argument is set to I(never), the running-config will
        never be copied to the startup-config.  If the argument is set to I(changed),
        then the running-config will only be copied to the startup-config if the task
        has made a change. I(changed) was added in Ansible 2.5.
    default: never
    choices:
      - always
      - never
      - modified
      - changed
    type: str
  diff_against:
    description:
      - When using the C(ansible-playbook --diff) command line argument the module can
        generate diffs against different sources.
      - When this option is configure as I(startup), the module will return the diff
        of the running-config against the startup-config.
      - When this option is configured as I(intended), the module will return the diff
        of the running-config against the configuration provided in the C(intended_config)
        argument.
      - When this option is configured as I(running), the module will return the before
        and after diff of the running-config with respect to any changes made to the
        device configuration.
    type: str
    choices:
      - running
      - startup
      - intended
  diff_ignore_lines:
    description:
      - Use this argument to specify one or more lines that should be ignored during
        the diff.  This is used for lines in the configuration that are automatically
        updated by the system.  This argument takes a list of regular expressions or
        exact line matches.
    type: list
    elements: str
  intended_config:
    description:
      - The C(intended_config) provides the master configuration that the node should
        conform to and is used to check the final running-config against. This argument
        will not modify any settings on the remote device and is strictly used to check
        the compliance of the current device's configuration against.  When specifying
        this argument, the task should also modify the C(diff_against) value and set
        it to I(intended). The configuration lines for this value should be similar to how it
        will appear if present in the running-configuration of the device including the indentation
        to ensure correct diff.
    type: str
  backup_options:
    description:
      - This is a dict object containing configurable options related to backup file
        path. The value of this option is read only when C(backup) is set to I(yes),
        if C(backup) is set to I(no) this option will be silently ignored.
    suboptions:
      filename:
        description:
          - The filename to be used to store the backup configuration. If the filename
            is not given it will be generated based on the hostname, current time and
            date in format defined by <hostname>_config.<current-date>@<current-time>
        type: str
      dir_path:
        description:
          - This option provides the path ending with directory name in which the backup
            configuration file will be stored. If the directory does not exist it will
            be first created and the filename is either the value of C(filename) or
            default filename as described in C(filename) options description. If the
            path value is not given in that case a I(backup) directory will be created
            in the current working directory and backup configuration will be copied
            in C(filename) within I(backup) directory.
        type: path
    type: dict
"""

EXAMPLES = """
- name: Configure top level configuration
  cisco.ios.ios_config:
    lines: hostname {{ inventory_hostname }}

- name: Configure interface settings
  cisco.ios.ios_config:
    lines:
      - description test interface
      - ip address 172.31.1.1 255.255.255.0
    parents: interface Ethernet1

- name: Configure ip helpers on multiple interfaces
  cisco.ios.ios_config:
    lines:
      - ip helper-address 172.26.1.10
      - ip helper-address 172.26.3.8
    parents: "{{ item }}"
  with_items:
    - interface Ethernet1
    - interface Ethernet2
    - interface GigabitEthernet1

- name: Configure policer in Scavenger class
  cisco.ios.ios_config:
    lines:
      - conform-action transmit
      - exceed-action drop
    parents:
      - policy-map Foo
      - class Scavenger
      - police cir 64000

- name: Load new acl into device
  cisco.ios.ios_config:
    lines:
      - 10 permit ip host 192.0.2.1 any log
      - 20 permit ip host 192.0.2.2 any log
      - 30 permit ip host 192.0.2.3 any log
      - 40 permit ip host 192.0.2.4 any log
      - 50 permit ip host 192.0.2.5 any log
    parents: ip access-list extended test
    before: no ip access-list extended test
    match: exact

- name: Check the running-config against master config
  cisco.ios.ios_config:
    diff_against: intended
    intended_config: "{{ lookup('file', 'master.cfg') }}"

- name: Check the startup-config against the running-config
  cisco.ios.ios_config:
    diff_against: startup
    diff_ignore_lines:
      - ntp clock .*

- name: Save running to startup when modified
  cisco.ios.ios_config:
    save_when: modified

- name: For idempotency, use full-form commands
  cisco.ios.ios_config:
    lines:
      # - shut
      - shutdown
    # parents: int gig1/0/11
    parents: interface GigabitEthernet1/0/11

# Set boot image based on comparison to a group_var (version) and the version
# that is returned from the `ios_facts` module
- name: Setting boot image
  cisco.ios.ios_config:
    lines:
      - no boot system
      - boot system flash bootflash:{{new_image}}
    host: "{{ inventory_hostname }}"
  when: ansible_net_version != version

- name: Render a Jinja2 template onto an IOS device
  cisco.ios.ios_config:
    backup: true
    src: ios_template.j2

- name: Configurable backup path
  cisco.ios.ios_config:
    src: ios_template.j2
    backup: true
    backup_options:
      filename: backup.cfg
      dir_path: /home/user

# Example ios_template.j2
# ip access-list extended test
#  permit ip host 192.0.2.1 any log
#  permit ip host 192.0.2.2 any log
#  permit ip host 192.0.2.3 any log
#  permit ip host 192.0.2.4 any log
"""

RETURN = """
updates:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['hostname foo', 'router ospf 1', 'router-id 192.0.2.1']
commands:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['hostname foo', 'router ospf 1', 'router-id 192.0.2.1']
backup_path:
  description: The full path to the backup file
  returned: when backup is yes
  type: str
  sample: /playbooks/ansible/backup/ios_config.2016-07-16@22:28:34
filename:
  description: The name of the backup file
  returned: when backup is yes and filename is not specified in backup options
  type: str
  sample: ios_config.2016-07-16@22:28:34
shortname:
  description: The full path to the backup file excluding the timestamp
  returned: when backup is yes and filename is not specified in backup options
  type: str
  sample: /playbooks/ansible/backup/ios_config
date:
  description: The date extracted from the backup file name
  returned: when backup is yes
  type: str
  sample: "2016-07-16"
time:
  description: The time extracted from the backup file name
  returned: when backup is yes
  type: str
  sample: "22:28:34"
"""
import json

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import ConnectionError
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
    dumps,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    get_config,
    get_connection,
    get_defaults_flag,
    run_commands,
)


def check_args(module, warnings):
    if module.params["multiline_delimiter"]:
        if len(module.params["multiline_delimiter"]) != 1:
            module.fail_json(msg="multiline_delimiter value can only be a single character")


def edit_config_or_macro(connection, commands):
    # only catch the macro configuration command,
    # not negated 'no' variation.
    if commands[0].startswith("macro name"):
        connection.edit_macro(candidate=commands)
    else:
        connection.edit_config(candidate=commands)


def get_candidate_config(module):
    candidate = ""
    if module.params["src"]:
        candidate = module.params["src"]
    elif module.params["lines"]:
        candidate_obj = NetworkConfig(indent=1)
        parents = module.params["parents"] or list()
        candidate_obj.add(module.params["lines"], parents=parents)
        candidate = dumps(candidate_obj, "raw")
    return candidate


def get_running_config(module, current_config=None, flags=None):
    running = module.params["running_config"]
    if not running:
        if not module.params["defaults"] and current_config:
            running = current_config
        else:
            running = get_config(module, flags=flags)
    return running


def save_config(module, result):
    result["changed"] = True
    if not module.check_mode:
        run_commands(module, "copy running-config startup-config\r")
    else:
        module.warn(
            "Skipping command `copy running-config startup-config` due to check_mode.  Configuration not copied to non-volatile storage",
        )


def main():
    """main entry point for module execution"""
    backup_spec = dict(filename=dict(), dir_path=dict(type="path"))
    argument_spec = dict(
        src=dict(type="str"),
        lines=dict(aliases=["commands"], type="list", elements="str"),
        parents=dict(type="list", elements="str"),
        before=dict(type="list", elements="str"),
        after=dict(type="list", elements="str"),
        match=dict(default="line", choices=["line", "strict", "exact", "none"]),
        replace=dict(default="line", choices=["line", "block"]),
        multiline_delimiter=dict(default="@"),
        running_config=dict(aliases=["config"]),
        intended_config=dict(),
        defaults=dict(type="bool", default=False),
        backup=dict(type="bool", default=False),
        backup_options=dict(type="dict", options=backup_spec),
        save_when=dict(choices=["always", "never", "modified", "changed"], default="never"),
        diff_against=dict(choices=["startup", "intended", "running"]),
        diff_ignore_lines=dict(type="list", elements="str"),
    )
    mutually_exclusive = [("lines", "src"), ("parents", "src")]
    required_if = [
        ("match", "strict", ["lines", "src"], True),
        ("match", "exact", ["lines", "src"], True),
        ("replace", "block", ["lines", "src"], True),
        ("diff_against", "intended", ["intended_config"]),
    ]
    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        required_if=required_if,
        supports_check_mode=True,
    )
    result = {"changed": False}
    warnings = list()
    check_args(module, warnings)
    result["warnings"] = warnings
    diff_ignore_lines = module.params["diff_ignore_lines"]
    config = None
    contents = None
    flags = get_defaults_flag(module) if module.params["defaults"] else []
    connection = get_connection(module)
    if module.params["backup"] or module._diff and module.params["diff_against"] == "running":
        contents = get_config(module, flags=flags)
        config = NetworkConfig(indent=1, contents=contents)
        if module.params["backup"]:
            result["__backup__"] = contents
    if any((module.params["lines"], module.params["src"])):
        match = module.params["match"]
        replace = module.params["replace"]
        path = module.params["parents"]
        candidate = get_candidate_config(module)
        running = get_running_config(module, contents, flags=flags)
        try:
            response = connection.get_diff(
                candidate=candidate,
                running=running,
                diff_match=match,
                diff_ignore_lines=diff_ignore_lines,
                path=path,
                diff_replace=replace,
            )
        except ConnectionError as exc:
            module.fail_json(msg=to_text(exc, errors="surrogate_then_replace"))
        config_diff = response["config_diff"]
        banner_diff = response["banner_diff"]
        if config_diff or banner_diff:
            commands = config_diff.split("\n")
            if module.params["before"]:
                commands[:0] = module.params["before"]
            if module.params["after"]:
                commands.extend(module.params["after"])
            result["commands"] = commands
            result["updates"] = commands
            result["banners"] = banner_diff

            # send the configuration commands to the device and merge
            # them with the current running config
            if not module.check_mode:
                if commands:
                    edit_config_or_macro(connection, commands)
                if banner_diff:
                    connection.edit_banner(
                        candidate=json.dumps(banner_diff),
                        multiline_delimiter=module.params["multiline_delimiter"],
                    )
            result["changed"] = True
    running_config = module.params["running_config"]
    startup_config = None
    if module.params["save_when"] == "always":
        save_config(module, result)
    elif module.params["save_when"] == "modified":
        output = run_commands(module, ["show running-config", "show startup-config"])
        running_config = NetworkConfig(indent=1, contents=output[0], ignore_lines=diff_ignore_lines)
        startup_config = NetworkConfig(indent=1, contents=output[1], ignore_lines=diff_ignore_lines)
        if running_config.sha1 != startup_config.sha1:
            save_config(module, result)
    elif module.params["save_when"] == "changed" and result["changed"]:
        save_config(module, result)
    if module._diff:
        if not running_config:
            output = run_commands(module, "show running-config")
            contents = output[0]
        else:
            contents = running_config

        # recreate the object in order to process diff_ignore_lines
        running_config = NetworkConfig(indent=1, contents=contents, ignore_lines=diff_ignore_lines)
        if module.params["diff_against"] == "running":
            if module.check_mode:
                module.warn("unable to perform diff against running-config due to check mode")
                contents = None
            else:
                contents = config.config_text
        elif module.params["diff_against"] == "startup":
            if not startup_config:
                output = run_commands(module, "show startup-config")
                contents = output[0]
            else:
                contents = startup_config.config_text
        elif module.params["diff_against"] == "intended":
            contents = module.params["intended_config"]
        if contents is not None:
            base_config = NetworkConfig(indent=1, contents=contents, ignore_lines=diff_ignore_lines)
            if running_config.sha1 != base_config.sha1:
                before, after = "", ""
                if module.params["diff_against"] == "intended":
                    before = running_config
                    after = base_config
                elif module.params["diff_against"] in ("startup", "running"):
                    before = base_config
                    after = running_config
                result.update(
                    {"changed": True, "diff": {"before": str(before), "after": str(after)}},
                )

    if result.get("changed") and any((module.params["src"], module.params["lines"])):
        msg = (
            "To ensure idempotency and correct diff the input configuration lines should be"
            " similar to how they appear if present in"
            " the running configuration on device"
        )
        if module.params["src"]:
            msg += " including the indentation"
        if "warnings" in result:
            result["warnings"].append(msg)
        else:
            result["warnings"] = msg

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: compliance_device_by_id_v1_info
short_description: Information module for Compliance Device By Id V1
description:
- Get all Compliance Device By Id V1.
- Return compliance detailed report for a device.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceUuid:
    description:
    - DeviceUuid path parameter. Device Id.
    type: str
  category:
    description:
    - >
      Category query parameter. Category can have any value among 'INTENT', 'RUNNING_CONFIG' , 'IMAGE' , 'PSIRT' ,
      'DESIGN_OOD' , 'EOX' , 'NETWORK_SETTINGS'.
    type: str
  complianceType:
    description:
    - >
      ComplianceType query parameter. Specify "Compliance type(s)" separated by commas. The Compliance type can be
      'APPLICATION_VISIBILITY', 'EOX', 'FABRIC', 'IMAGE', 'NETWORK_PROFILE', 'NETWORK_SETTINGS', 'PSIRT',
      'RUNNING_CONFIG', 'WORKFLOW'.
    type: str
  diffList:
    description:
    - DiffList query parameter. Diff list pass true to fetch the diff list.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Compliance ComplianceDetailsOfDeviceV1
  description: Complete reference of the ComplianceDetailsOfDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!compliance-details-of-device
notes:
  - SDK Method used are
    compliance.Compliance.compliance_details_of_device_v1,

  - Paths used are
    get /dna/intent/api/v1/compliance/{deviceUuid}/detail,

"""

EXAMPLES = r"""
- name: Get all Compliance Device By Id V1
  cisco.dnac.compliance_device_by_id_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    category: string
    complianceType: string
    diffList: True
    deviceUuid: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "deviceUuid": "string",
          "complianceType": "string",
          "status": "string",
          "state": "string",
          "lastSyncTime": 0,
          "lastUpdateTime": 0,
          "sourceInfoList": [
            {
              "name": "string",
              "nameWithBusinessKey": "string",
              "sourceEnum": "string",
              "type": "string",
              "appName": "string",
              "count": 0,
              "ackStatus": "string",
              "businessKey": {
                "resourceName": "string",
                "businessKeyAttributes": {},
                "otherAttributes": {
                  "name": "string",
                  "cfsAttributes": {
                    "displayName": "string",
                    "appName": "string",
                    "description": "string",
                    "source": "string",
                    "type": "string"
                  }
                }
              },
              "diffList": [
                {
                  "op": "string",
                  "configuredValue": "string",
                  "intendedValue": "string",
                  "moveFromPath": "string",
                  "businessKey": "string",
                  "path": "string",
                  "extendedAttributes": {
                    "attributeDisplayName": "string",
                    "path": "string",
                    "dataConverter": "string",
                    "type": "string"
                  },
                  "ackStatus": "string",
                  "instanceUUID": "string",
                  "displayName": "string"
                }
              ],
              "displayName": "string"
            }
          ],
          "ackStatus": "string",
          "version": "string"
        }
      ],
      "deviceUuid": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the iosxr_ospf_interfaces module
"""


class Ospf_interfacesArgs(object):  # pylint: disable=R0903
    """The arg spec for the iosxr_ospf_interfaces module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "type": {"type": "str", "required": True},
                "address_family": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "afi": {
                            "type": "str",
                            "choices": ["ipv4", "ipv6"],
                            "required": True,
                        },
                        "processes": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "process_id": {
                                    "type": "str",
                                    "required": True,
                                },
                                "area": {
                                    "type": "dict",
                                    "options": {"area_id": {"type": "str"}},
                                },
                            },
                        },
                        "apply_group_option": {
                            "type": "dict",
                            "options": {
                                "group_name": {"type": "str"},
                                "operation": {
                                    "type": "str",
                                    "choices": ["add", "remove", "append"],
                                },
                            },
                        },
                        "authentication": {
                            "no_log": False,
                            "type": "dict",
                            "options": {
                                "message_digest": {
                                    "no_log": False,
                                    "type": "dict",
                                    "options": {
                                        "keychain": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                    },
                                },
                                "null_auth": {"type": "bool"},
                            },
                        },
                        "authentication_key": {
                            "type": "dict",
                            "no_log": True,
                            "options": {
                                "password": {"type": "str", "no_log": True},
                                "clear": {"type": "str"},
                                "encrypted": {"type": "str"},
                            },
                        },
                        "bfd": {
                            "type": "dict",
                            "options": {
                                "fast_detect": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "strict_mode": {"type": "bool"},
                                    },
                                },
                                "minimum_interval": {"type": "int"},
                                "multiplier": {"type": "int"},
                            },
                        },
                        "cost": {"type": "int"},
                        "cost_fallback": {
                            "type": "dict",
                            "options": {
                                "cost": {"type": "int"},
                                "threshold": {"type": "int"},
                            },
                        },
                        "database_filter": {
                            "type": "dict",
                            "options": {"all_outgoing_lsa": {"type": "bool"}},
                        },
                        "dead_interval": {"type": "int"},
                        "demand_circuit": {"type": "bool"},
                        "fast_reroute": {
                            "type": "dict",
                            "options": {
                                "disabled": {"type": "bool"},
                                "per_link": {
                                    "type": "dict",
                                    "options": {
                                        "information_type": {
                                            "type": "str",
                                            "choices": [
                                                "exclude",
                                                "lfa_candidate",
                                            ],
                                        },
                                        "use_candidate_only": {"type": "bool"},
                                        "interface": {
                                            "type": "dict",
                                            "options": {
                                                "bvi": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "bundle_ether": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "pos_int": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "fast_ethernet": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "fiftygige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "fortygige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "fourhundredgige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "gigabitethernet": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "hundredgige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "mgmteth": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "multilink": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "pw_ether": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "pw_iw": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "srp": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "serial": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "tengige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "twentyfivegige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "twohundredgige": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "nve": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "tunnel_ip": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "tunnel_ipsec": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "tunnel_mte": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                                "tunnel_mpls": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "flood_reduction": {"type": "bool"},
                        "hello_interval": {"type": "int"},
                        "link_down_fast_detect": {"type": "bool"},
                        "message_digest_key": {
                            "type": "dict",
                            "no_log": False,
                            "options": {
                                "id": {"type": "int", "required": True},
                                "md5": {
                                    "type": "dict",
                                    "required": True,
                                    "options": {
                                        "password": {
                                            "type": "str",
                                            "no_log": True,
                                        },
                                        "clear": {"type": "bool"},
                                        "encrypted": {"type": "bool"},
                                    },
                                },
                            },
                        },
                        "mpls_ldp_sync": {"type": "bool"},
                        "mtu_ignore": {"type": "bool"},
                        "network": {
                            "type": "str",
                            "choices": [
                                "broadcast",
                                "non-broadcast",
                                "point-to-multipoint",
                                "point-to-point",
                            ],
                        },
                        "neighbors": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "neighbor_id": {"type": "str"},
                                "cost": {"type": "int"},
                                "db_filter_all_out": {"type": "bool"},
                                "poll_interval": {"type": "int"},
                                "priority": {"type": "int"},
                            },
                        },
                        "packet_size": {"type": "int"},
                        "passive": {"type": "bool"},
                        "prefix_suppression": {"type": "bool"},
                        "priority": {"type": "int"},
                        "retransmit_interval": {"type": "int"},
                        "security_ttl": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "hops": {"type": "int"},
                            },
                        },
                        "transmit_delay": {"type": "int"},
                    },
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "parsed",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_user_defined_field_update_v1
short_description: Resource module for Network Device User Defined Field Update V1
description:
- Manage operation update of the resource Network Device User Defined Field Update V1.
- >
   Assigns an existing Global User-Defined-Field to a device. If the UDF is already assigned to the specific device,
   then it updates the device UDF value accordingly. Please note that the assigning UDF 'name' must be an existing
   global UDF. Otherwise error shall be shown.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceId:
    description: DeviceId path parameter. UUID of device to which UDF has to be added.
    type: str
  payload:
    description: Network Device User Defined Field Update's payload.
    elements: dict
    suboptions:
      name:
        description: Name of the User Defined Field.
        type: str
      value:
        description: Value of the User Defined Field that will be assigned to the device.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices AddUserDefinedFieldToDeviceV1
  description: Complete reference of the AddUserDefinedFieldToDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-user-defined-field-to-device
notes:
  - SDK Method used are
    devices.Devices.add_user_defined_field_to_device_v1,

  - Paths used are
    put /dna/intent/api/v1/network-device/{deviceId}/user-defined-field,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.network_device_user_defined_field_update_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceId: string
    payload:
    - name: string
      value: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos_route_maps config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.route_maps import (
    Route_mapsTemplate,
)


class Route_maps(ResourceModule):
    """
    The eos_route_maps config class
    """

    def __init__(self, module):
        super(Route_maps, self).__init__(
            empty_fact_val=[],
            facts_module=Facts(module),
            module=module,
            resource="route_maps",
            tmplt=Route_mapsTemplate(),
        )
        self.parsers = [
            "continue",
            "route_map.copy",
            "route_map.rename",
            "description",
            "sub_route_map",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {}
        haved = {}

        for entry in self.want:
            wantd.update({entry["route_map"]: entry})
        for entry in self.have:
            haved.update({entry["route_map"]: entry})

        # turn all lists of dicts into dicts prior to merge
        for entry in wantd, haved:
            self._route_maps_list_to_dict(entry)
        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            h_del = {}
            for k, v in iteritems(haved):
                if k in wantd or not wantd:
                    h_del.update({k: v})
            haved = h_del
            for rmap, val in iteritems(haved):
                self.addcmd({"route_map": rmap}, "route_map.name", True)
            wantd = {}

        # remove superfluous config for overridden
        if self.state in ["overridden"]:
            for k, have in iteritems(haved):
                for entry, val in iteritems(have.get("entries", {})):
                    if not wantd.get(k) or entry not in wantd[k].get(
                        "entries",
                        {},
                    ):
                        self._compare_maps(want={}, have={entry: val})

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Route_maps network resource.
        """
        self._compare_entries(want=want, have=have)

    def _compare_entries(self, want, have):
        w_entries = want.get("entries", {})
        h_entries = have.get("entries", {})
        # overridden
        if not w_entries:
            for k, v in iteritems(h_entries):
                self._compare_maps({}, {k: v})
        for k, v in iteritems(w_entries):
            before_maps = len(self.commands)
            self._compare_maps({k: v}, {k: h_entries.get(k, {})})
            after_maps = len(self.commands)
            self._compare_match(v, h_entries.get(k, {}))
            self._comapre_set(v, h_entries.get(k, {}))
            for entry_k, entry_v in iteritems(v):
                h = {}
                if h_entries.get(k):
                    h = {"entries": {entry_k: h_entries[k].pop(entry_k, {})}}
                self.compare(
                    parsers=self.parsers,
                    want={"entries": {entry_k: entry_v}},
                    have=h,
                )
            for h_k, h_v in iteritems(h_entries.pop(k, {})):
                self.compare(
                    parsers=self.parsers,
                    have={"entries": {h_k: h_v}},
                    want={},
                )

            parent_present = False
            for c in self.commands[before_maps::]:
                if c.startswith("route-map"):
                    parent_present = True
                    break
            if (before_maps == after_maps and len(self.commands) > after_maps) or (
                not parent_present and len(self.commands) > after_maps
            ):
                self._compare_maps({k: v}, {})
                self.commands.insert(after_maps, self.commands.pop(-1))

    def _compare_maps(self, want, have):
        map_in_want = []
        for k, v in iteritems(want):
            map_name = k.split(" ")[0]
            map_in_want.append(map_name)
            w = {}
            h = {}
            h_entry = {}
            for entry_k, entry_v in iteritems(v):
                if entry_k not in [
                    "continue_sequence",
                    "sub_route_map",
                    "description",
                    "match",
                    "set",
                ]:
                    w.update({entry_k: entry_v})
                    if have.get(k):
                        h.update({entry_k: have[k].pop(entry_k, {})})
            if h:
                h_entry = {"route_map": map_name, "entries": h}
            parser = self._select_parser(w)
            self.compare(
                parsers=parser,
                want={"route_map": map_name, "entries": w},
                have=h_entry,
            )
        for k, v in iteritems(have):
            map_name = k.split(" ")[0]
            if k not in want.keys() and self.state in [
                "replaced",
                "overridden",
            ]:
                w_negate = {}
                if map_name not in map_in_want and self.state == "replaced":
                    continue
                parser = self._select_parser(v)
                w_negate.update({"route_map": map_name, "entries": v})
                self.addcmd(w_negate, parser, True)

    def _select_parser(self, w):
        parser = ""
        if "statement" in w.keys() and "action" in w.keys() and "sequence" in w.keys():
            parser = "route_map.statement.entries"
        elif "statement" in w.keys() and "action" in w.keys():
            parser = "route_map.statement.action"
        elif "statement" in w.keys():
            parser = "route_map.statement.name"
        elif "action" in w.keys() and "sequence" in w.keys():
            parser = "route_map.entries"
        elif "action" in w.keys():
            parser = "route_map.action"
        else:
            parser = "route_map.name"
        return parser

    def _compare_match(self, want, have):
        parsers = [
            "match.aggregate_role",
            "match.as",
            "match.as_path",
            "match.community.instances",
            "match.community.list",
            "match.extcommunity",
            "match.invert.aggregate_role",
            "match.invert.as_path",
            "match.invert.community.instances",
            "match.invert.community.list",
            "match.invert.extcommunity",
            "match.interface",
            "match.ip",
            "match.ipaddress",
            "match.ipv6",
            "match.ipv6address",
            "match.largecommunity",
            "match.isis",
            "match.local_pref",
            "match.metric",
            "match.metric_type",
            "match.route_type",
            "match.routerid",
            "match.source_protocol",
            "match.tag",
        ]
        w_match = want.pop("match", {})
        h_match = have.pop("match", {})
        for k, v in iteritems(w_match):
            if k in ["ip", "ipv6"]:
                for k_ip, v_ip in iteritems(v):
                    if h_match.get(k):
                        h = {k_ip: h_match[k].pop(k_ip, {})}
                    else:
                        h = {}
                    self.compare(
                        parsers=[
                            "match.ip",
                            "match.ipaddress",
                            "match.ipv6address",
                            "match.ipv6",
                        ],
                        want={"entries": {"match": {k: {k_ip: v_ip}}}},
                        have={"entries": {"match": {k: h}}},
                    )
                h_match.pop(k, {})
                continue
            self.compare(
                parsers=parsers,
                want={"entries": {"match": {k: v}}},
                have={"entries": {"match": {k: h_match.pop(k, {})}}},
            )
        for k, v in iteritems(h_match):
            if k in ["ip", "ipv6"]:
                for hk, hv in iteritems(v):
                    self.compare(
                        parsers=[
                            "match.ip",
                            "match.ipaddress",
                            "match.ipv6address",
                            "match.ipv6",
                        ],
                        want={},
                        have={"entries": {"match": {k: {hk: hv}}}},
                    )
                continue
            self.compare(
                parsers=parsers,
                want={},
                have={"entries": {"match": {k: v}}},
            )

    def _comapre_set(self, want, have):
        parsers = [
            "set.as_path.prepend",
            "set.as_path.match",
            "set.bgp",
            "set.community.graceful_shutdown",
            "set.community.none",
            "set.community.number",
            "set.community.list",
            "set.community.internet",
            "set.distance",
            "set.evpn",
            "set.extcommunity.lbw",
            "set.extcommunity.none",
            "set.extcommunity.rt",
            "set.extcommunity.soo",
            "set.ip",
            "set.ipv6",
            "set.isis",
            "set.local_pref",
            "set.metric.value",
            "set.metric_type",
            "set.nexthop",
            "set.origin",
            "set.segment_index",
            "set.tag",
            "set.weight",
        ]

        w_set = want.pop("set", {})
        h_set = have.pop("set", {})
        for k, v in iteritems(w_set):
            self.compare(
                parsers=parsers,
                want={"entries": {"set": {k: v}}},
                have={"entries": {"set": {k: h_set.pop(k, {})}}},
            )
        for k, v in iteritems(h_set):
            self.compare(
                parsers=parsers,
                want={},
                have={"entries": {"set": {k: v}}},
            )

    def _route_maps_list_to_dict(self, entry):
        for name, r_map in iteritems(entry):
            if r_map.get("entries"):
                map_dict = {}
                for entry in r_map["entries"]:
                    if entry.get("sequence"):
                        seq = entry["sequence"]
                    else:
                        seq = "seq"
                    mapkey = name + " " + str(seq)
                    map_dict.update({mapkey: entry})
                r_map["entries"] = map_dict



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sqlmidbshorttermretentionpolicy
version_added: "2.4.0"
short_description: Manage SQL Managed Instance database backup short term retention policy
description:
    - Manage SQL Managed Instance database backup short term retention policy.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
        type: str
        required: true
    managed_instance_name:
        description:
            - The name of the SQL managed instance.
        type: str
        required: true
    database_name:
        description:
            - The name of the SQL managed instance database.
        type: str
        required: true
    policy_name:
        description:
            - The name of the SQL managed instance short term retention policy.
        type: str
        required: true
        choices:
            - default
    retention_days:
        description:
            - The backup retention period in days. This is how many days Point-in-Time.
        type: int
        default: 7

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Update SQL managed instance short term retention policy's retention_days
  azure_rm_sqlmidbshorttermretentionpolicy:
    resource_group: testrg
    managed_instance_name: testinstancename
    database_name: newdatabase
    policy_name: default
    retention_days: 3
'''

RETURN = '''
short_term_retention_policy:
    description:
        - A dictionary containing facts for SQL Managed Instance Short Term Retention Policies.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Sql/
                     managedInstances/fredsqlmi/databases/newdatabase/backupShortTermRetentionPolicies/default"
        database_name:
            description:
                - SQL managed instance database name.
            returned: always
            type: str
            sample: newdatabase
        policy_name:
            description:
                - SQL managed instance short term retentioni policy name.
            returned: always
            type: str
            sample: default
        managed_instance_name:
            description:
                - SQL managed instance name.
            returned: always
            type: str
            sample: testmanagedinstance
        type:
            description:
                - The SQL managed instance type.
            type: str
            returned: always
            sample: "Microsoft.Sql/managedInstances"
        resource_group:
            description:
                - The resource relate resource group.
            type: str
            returned: always
            sample: testRG
        retention_days:
            description:
                - The backup retention period in days. This is how many days Point-in-Time.
            type: int
            sample: 7
            returned: always
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMSqMIShortTermRetentionPolicy(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            managed_instance_name=dict(
                type='str',
                required=True,
            ),
            database_name=dict(
                type='str',
                required=True,
            ),
            policy_name=dict(
                type='str',
                required=True,
                choices=['default']
            ),
            retention_days=dict(
                type='int',
                default=7
            ),
        )
        # store the results of the module operation
        self.results = dict(
            changed=False,
            diff=[]
        )
        self.resource_group = None
        self.managed_instance_name = None
        self.database_name = None
        self.policy_name = None
        self.retention_days = None

        super(AzureRMSqMIShortTermRetentionPolicy, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        old_response = self.get()

        if old_response is not None:
            if self.retention_days is not None and old_response['retention_days'] != self.retention_days:
                self.results['changed'] = True
                self.results['diff'].append('retention_days')
                if not self.check_mode:
                    self.results['short_term_retention_policy'] = self.update_policy()
        else:
            self.results['changed'] = True
            if not self.check_mode:
                self.results['short_term_retention_policy'] = self.create_policy()
        return self.results

    def get(self):
        response = None
        try:
            response = self.sql_client.managed_backup_short_term_retention_policies.get(resource_group_name=self.resource_group,
                                                                                        managed_instance_name=self.managed_instance_name,
                                                                                        database_name=self.database_name,
                                                                                        policy_name=self.policy_name)
            self.log("Response : {0}".format(response))
        except HttpResponseError:
            self.log('Could not get facts for SQL managed instance short term retention policyes.')

        return self.format_item(response) if response is not None else None

    def update_policy(self):
        response = None
        try:
            response = self.sql_client.managed_backup_short_term_retention_policies.begin_update(resource_group_name=self.resource_group,
                                                                                                 managed_instance_name=self.managed_instance_name,
                                                                                                 database_name=self.database_name,
                                                                                                 policy_name=self.policy_name,
                                                                                                 parameters=dict(retention_days=self.retention_days))
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.fail('Could not update the SQL managed instance short term retention policyes. Exception as {0}'.format(ec))

        return self.format_item(self.get_poller_result(response))

    def create_policy(self):
        response = None
        try:
            response = self.sql_client.managed_backup_short_term_retention_policies.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                                           managed_instance_name=self.managed_instance_name,
                                                                                                           database_name=self.database_name,
                                                                                                           policy_name=self.policy_name,
                                                                                                           parameters=dict(retention_days=self.retention_days))
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.fail('Could not Create the SQL managed instance short term retention policyes. Exception as {0}'.format(ec))

        return self.format_item(self.get_poller_result(response))

    def format_item(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'managed_instance_name': self.managed_instance_name,
            'database_name': self.database_name,
            'id': d.get('id', None),
            'name': d.get('name', None),
            'type': d.get('type', None),
            'retention_days': d.get('retention_days', None),
        }
        return d


def main():
    AzureRMSqMIShortTermRetentionPolicy()


if __name__ == '__main__':
    main()



# this is a virtual module that is entirely implemented server side
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: telnet
short_description: Executes a low-down and dirty telnet command
description:
  - Executes a low-down and dirty telnet command, not going through the module subsystem.
  - This is mostly to be used for enabling ssh on devices that only have telnet enabled
    by default.
version_added: 1.0.0
options:
  command:
    description:
      - List of commands to be executed in the telnet session.
    required: true
    type: list
    elements: str
    aliases:
      - commands
  host:
    description:
      - The host/target on which to execute the command
    required: false
    type: str
    default: remote_addr
  user:
    description:
      - The user for login
    required: false
    type: str
    default: remote_user
  password:
    description:
      - The password for login
    type: str
  port:
    description:
      - Remote port to use
    type: int
    default: 23
  timeout:
    description:
      - timeout for remote operations
    type: int
    default: 120
  prompts:
    description:
      - List of prompts expected before sending next command
    required: false
    type: list
    elements: str
    default:
      - $
  login_prompt:
    description:
      - Login or username prompt to expect
    required: false
    type: str
    default: "login: "
  password_prompt:
    description:
      - Login or username prompt to expect
    required: false
    type: str
    default: "Password: "
  pause:
    description:
      - Seconds to pause between each command issued
    required: false
    type: int
    default: 1
  send_newline:
    description:
      - Sends a newline character upon successful connection to start the terminal session.
    required: false
    type: bool
    default: false
  crlf:
    description:
      - Sends a CRLF (Carrage Return) instead of just a LF (Line Feed).
    required: false
    type: bool
    default: false
notes:
  - The C(environment) keyword does not work with this task
author:
  - Ansible Core Team
"""

EXAMPLES = """
- name: send configuration commands to IOS
  ansible.netcommon.telnet:
    user: cisco
    password: cisco
    login_prompt: "Username: "
    prompts:
      - "[>#]"
    command:
      - terminal length 0
      - configure terminal
      - hostname ios01

- name: run show commands
  ansible.netcommon.telnet:
    user: cisco
    password: cisco
    login_prompt: "Username: "
    prompts:
      - "[>#]"
    command:
      - terminal length 0
      - show version
"""

RETURN = """
output:
  description: output of each command is an element in this list
  type: list
  returned: always
  sample: ["success", "success", "", "warning .. something"]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_access_dictionary_info
short_description: Information module for Network Access Dictionary
description:
- Get all Network Access Dictionary.
- Get Network Access Dictionary by name.
- GET a dictionary by name.
- Get all Dictionaries.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  name:
    description:
    - Name path parameter. The dictionary name.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Network Access - Dictionary
  description: Complete reference of the Network Access - Dictionary API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!policy-openapi
notes:
  - SDK Method used are
    network_access_dictionary.NetworkAccessDictionary.get_network_access_dictionaries,
    network_access_dictionary.NetworkAccessDictionary.get_network_access_dictionary_by_name,

  - Paths used are
    get /network-access/dictionaries,
    get /network-access/dictionaries/{name},

"""

EXAMPLES = r"""
- name: Get all Network Access Dictionary
  cisco.ise.network_access_dictionary_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

- name: Get Network Access Dictionary by name
  cisco.ise.network_access_dictionary_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    name: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "description": "string",
      "dictionaryAttrType": "string",
      "id": "string",
      "link": {
        "href": "string",
        "rel": "string",
        "type": "string"
      },
      "name": "string",
      "version": "string"
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
    ise_hostname:
        description:
          - The Identity Services Engine hostname.
        type: str
        required: true
    ise_username:
        description:
          - The Identity Services Engine username to authenticate.
        type: str
        required: true
    ise_password:
        description:
          - The Identity Services Engine password to authenticate.
        type: str
        required: true
    ise_verify:
        description:
          - Flag to enable or disable SSL certificate verification.
        type: bool
        default: true
    ise_version:
        description:
          - Informs the SDK which version of Identity Services Engine to use.
        type: str
        default: 3.1_Patch_1
    ise_wait_on_rate_limit:
        description:
          - Flag for Identity Services Engine SDK to enable automatic rate-limit handling.
        type: bool
        default: true
    ise_debug:
        description:
          - Flag for Identity Services Engine SDK to enable debugging.
        type: bool
        default: false
    ise_uses_api_gateway:
        description:
          - Flag that informs the SDK whether to use the Identity Services Engine's API Gateway to send requests.
          - If it is true, it uses the ISE's API Gateway and sends requests to https://{{ise_hostname}}.
          - If it is false, it sends the requests to https://{{ise_hostname}}:{{port}}, where the port value depends on the Service used (ERS, Mnt, UI, PxGrid).
        type: bool
        default: true
        version_added: '1.1.0'
    ise_uses_csrf_token:
        description:
          - Flag that informs the SDK whether we send the CSRF token to ISE's ERS APIs.
          - If it is True, the SDK assumes that your ISE CSRF Check is enabled.
          - If it is True, it assumes you need the SDK to manage the CSRF token automatically for you.
        type: bool
        default: false
        version_added: '3.0.0'
    ise_single_request_timeout:
        description:
          -  Timeout (in seconds) for RESTful HTTP requests.
        type: int
        default: 60
        version_added: '3.0.0'
notes:
    - "Supports C(check_mode)"
    - "The plugin runs on the control node and does not use any ansible connection plugins, but instead the embedded connection manager from Cisco ISE SDK"
    - "The parameters starting with ise_ are used by the Cisco ISE Python SDK to establish the connection"
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mnt_athentication_status_info
short_description: Information module for Mnt Athentication Status
description:
- Get all Mnt Athentication Status.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  MAC:
    description:
    - MAC path parameter.
    type: str
  SECONDS:
    description:
    - SECONDS path parameter.
    type: str
  RECORDS:
    description:
    - RECORDS path parameter.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    misc.Misc.get_authentication_status_by_mac,

  - Paths used are
    get /AuthStatus/MACAddress/{MAC}/{SECONDS}/{RECORDS}/All,
seealso:
# Reference by module name
- module: cisco.ise.mnt_authentication_status_info
"""

EXAMPLES = r"""
- name: Get all Mnt Athentication Status
  cisco.ise.mnt_athentication_status_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    MAC: string
    SECONDS: string
    RECORDS: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_provision_device_v1
short_description: Resource module for Sda Provision Device V1
description:
- Manage operations create, update and delete of the resource Sda Provision Device V1.
- Provision Wired Device.
- Delete provisioned Wired Device.
- Re-Provision Wired Device.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceManagementIpAddress:
    description: DeviceManagementIpAddress query parameter. Valid IP address of the
      device currently provisioned in a fabric site.
    type: str
  siteNameHierarchy:
    description: SiteNameHierarchy of the provisioned device.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA ProvisionWiredDeviceV1
  description: Complete reference of the ProvisionWiredDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!provision-wired-device
- name: Cisco DNA Center documentation for SDA DeleteProvisionedWiredDeviceV1
  description: Complete reference of the DeleteProvisionedWiredDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-provisioned-wired-device
- name: Cisco DNA Center documentation for SDA ReProvisionWiredDeviceV1
  description: Complete reference of the ReProvisionWiredDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!re-provision-wired-device
notes:
  - SDK Method used are
    sda.Sda.delete_provisioned_wired_device_v1,
    sda.Sda.provision_wired_device_v1,
    sda.Sda.re_provision_wired_device_v1,

  - Paths used are
    post /dna/intent/api/v1/business/sda/provision-device,
    delete /dna/intent/api/v1/business/sda/provision-device,
    put /dna/intent/api/v1/business/sda/provision-device,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.sda_provision_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    deviceManagementIpAddress: string

- name: Update all
  cisco.dnac.sda_provision_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceManagementIpAddress: string
    siteNameHierarchy: string

- name: Create
  cisco.dnac.sda_provision_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceManagementIpAddress: string
    siteNameHierarchy: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: security_rogue_additional_details
short_description: Resource module for Security Rogue Additional Details
description:
- This module represents an alias of the module security_rogue_additional_details_v1
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: This is the epoch end time in milliseconds upto which data need to
      be fetched. Default value is current time.
    type: float
  limit:
    description: The maximum number of entries to return. Default value is 1000.
    type: float
  offset:
    description: The offset of the first item in the collection to return. Default value
      is 1.
    type: float
  siteId:
    description: Filter Rogues by location. Site IDs information can be fetched from
      "Get Site" API.
    elements: str
    type: list
  startTime:
    description: This is the epoch start time in milliseconds from which data need to
      be fetched. Default value is 24 hours earlier to endTime.
    type: float
  threatLevel:
    description: Filter Rogues by Threat Level. Threat Level information can be fetched
      from "Get Threat Levels" API.
    elements: str
    type: list
  threatType:
    description: Filter Rogues by Threat Type. Threat Type information can be fetched
      from "Get Threat Types" API.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices RogueAdditionalDetailsV1
  description: Complete reference of the RogueAdditionalDetailsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!rogue-additional-details
notes:
  - SDK Method used are
    devices.Devices.rogue_additional_details_v1,

  - Paths used are
    post /dna/intent/api/v1/security/rogue/additional/details,
  - It should be noted that this module is an alias of security_rogue_additional_details_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.security_rogue_additional_details:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    endTime: 0
    limit: 0
    offset: 0
    siteId:
    - string
    startTime: 0
    threatLevel:
    - string
    threatType:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "macAddress": "string",
          "mldMacAddress": "string",
          "updatedTime": 0,
          "createdTime": 0,
          "threatType": "string",
          "threatLevel": "string",
          "apName": "string",
          "detectingAPMac": "string",
          "ssid": "string",
          "containment": "string",
          "radioType": "string",
          "controllerIp": "string",
          "controllerName": "string",
          "channelNumber": "string",
          "siteNameHierarchy": "string",
          "encryption": "string",
          "switchIp": "string",
          "switchName": "string",
          "portDescription": "string"
        }
      ],
      "totalCount": 0,
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The asa_ogs class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import copy

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.asa.plugins.module_utils.network.asa.facts.facts import Facts
from ansible_collections.cisco.asa.plugins.module_utils.network.asa.rm_templates.ogs import (
    OGsTemplate,
)


class OGs(ResourceModule):
    """
    The asa_ogs class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["ogs"]

    def __init__(self, module):
        super(OGs, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ogs",
            tmplt=OGsTemplate(),
        )

    def execute_module(self):
        """Execute the module
        :rtype: A dictionary
        :returns: The result from module execution
        """
        self.gen_config()
        self.run_commands()
        return self.result

    def gen_config(self):
        """Select the appropriate function based on the state provided
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        if self.want:
            temp = {}
            for entry in self.want:
                temp.update({(entry["object_type"]): entry})
            wantd = temp
        else:
            wantd = {}
        if self.have:
            temp = {}
            for entry in self.have:
                temp.update({(entry["object_type"]): entry})
            haved = temp
        else:
            haved = {}

        obj_gp = {}
        for k, v in wantd.items():
            temp = {}
            for each in v.get("object_groups"):
                temp[each.get("name")] = each
                temp["object_type"] = k
                obj_gp[k] = temp
        if obj_gp:
            wantd = obj_gp
            obj_gp = {}
        for k, v in haved.items():
            temp = {}
            for each in v.get("object_groups"):
                temp[each.get("name")] = each
                temp["object_type"] = k
                obj_gp[k] = temp
        if obj_gp:
            haved = obj_gp

        # if state is merged, merge want onto have
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            temp = {}
            for k, v in iteritems(haved):
                temp_have = {}
                if k in wantd or not wantd:
                    for key, val in iteritems(v):
                        if not wantd or key in wantd[k]:
                            temp_have.update({key: val})
                    temp.update({k: temp_have})
            haved = temp
            wantd = {}

        # delete processes first so we do run into "more than one" errors
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    for each_key, each_val in iteritems(have):
                        if each_key != "object_type":
                            each_val.update(
                                {"object_type": have.get("object_type")},
                            )
                            self.addcmd(each_val, "og_name", True)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        if want != have:
            for k, v in iteritems(want):
                if k != "object_type":
                    v.update({"object_type": want.get("object_type")})
            if have:
                for k, v in iteritems(have):
                    if k != "object_type":
                        v.update({"object_type": want.get("object_type")})

            object_type = want.get("object_type")
            if object_type == "icmp-type":
                self._icmp_object_compare(want, have)
            if object_type == "network":
                self._network_object_compare(want, have)
            elif object_type == "protocol":
                self._protocol_object_compare(want, have)
            elif object_type == "security":
                self._security_object_compare(want, have)
            elif object_type == "service":
                self._service_object_compare(want, have)
            elif object_type == "user":
                self._user_object_compare(want, have)

    def get_list_diff(self, want, have, object, param):
        diff = [item for item in want[object][param] if item not in have[object][param]]
        return diff

    def check_for_have_and_overidden(self, have):
        if have and self.state == "overridden":
            for name, entry in iteritems(have):
                if name != "object_type":
                    self.addcmd(entry, "og_name", True)

    def _icmp_object_compare(self, want, have):
        icmp_obj = "icmp_type"
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type" and entry[icmp_obj].get("icmp_object"):
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        icmp_obj,
                        ["icmp_type"],
                    )
                else:
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if self.state in ("overridden", "replaced") and h_item:
                    self.compare(["icmp_type"], {}, h_item)
                if h_item and h_item[icmp_obj].get("icmp_object"):
                    li_diff = self.get_list_diff(
                        entry,
                        h_item,
                        icmp_obj,
                        "icmp_object",
                    )
                else:
                    li_diff = entry[icmp_obj].get("icmp_object")
                entry[icmp_obj]["icmp_object"] = li_diff
                self.addcmd(entry, "icmp_type", False)
        self.check_for_have_and_overidden(have)

    def _network_object_compare(self, want, have):
        network_obj = "network_object"
        parsers = [
            "network_object.host",
            "network_object.address",
            "network_object.ipv6_address",
            "network_object.object",
        ]
        add_obj_cmd = False
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type":
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        network_obj,
                        ["address", "host", "ipv6_address", "object"],
                    )
                else:
                    add_obj_cmd = True
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if entry[network_obj].get("address"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        network_obj,
                        "address",
                        parsers,
                        "network_object.address",
                    )
                elif h_item and h_item.get(network_obj) and h_item[network_obj].get("address"):
                    h_item[network_obj] = {
                        "address": h_item[network_obj].get("address"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
                if entry[network_obj].get("host"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        network_obj,
                        "host",
                        parsers,
                        "network_object.host",
                    )
                elif h_item and h_item[network_obj].get("host"):
                    h_item[network_obj] = {
                        "host": h_item[network_obj].get("host"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
                if entry[network_obj].get("ipv6_address"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        network_obj,
                        "ipv6_address",
                        parsers,
                        "network_object.ipv6_address",
                    )
                elif h_item and h_item.get(network_obj) and h_item[network_obj].get("ipv6_address"):
                    h_item[network_obj] = {
                        "ipv6_address": h_item[network_obj].get("ipv6_address"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
                if entry[network_obj].get("object"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        network_obj,
                        "object",
                        parsers,
                        "network_object.object",
                    )
                elif h_item and h_item.get(network_obj) and h_item[network_obj].get("object"):
                    h_item[network_obj] = {
                        "object": h_item[network_obj].get("object"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
        self.check_for_have_and_overidden(have)

    def _protocol_object_compare(self, want, have):
        protocol_obj = "protocol_object"
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type":
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        protocol_obj,
                        ["protocol"],
                    )
                else:
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if entry[protocol_obj].get("protocol"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        protocol_obj,
                        "protocol",
                        [protocol_obj],
                        protocol_obj,
                    )
        self.check_for_have_and_overidden(have)

    def _security_object_compare(self, want, have):
        security_obj = "security_group"
        parsers = ["security_group.sec_name", "security_group.tag"]
        add_obj_cmd = False
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type":
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        security_obj,
                        ["sec_name", "tag"],
                    )
                else:
                    add_obj_cmd = True
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if entry[security_obj].get("sec_name"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        security_obj,
                        "sec_name",
                        parsers,
                        "security_group.sec_name",
                    )
                elif h_item and h_item[security_obj].get("sec_name"):
                    h_item[security_obj] = {
                        "sec_name": h_item[security_obj].get("sec_name"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
                if entry[security_obj].get("tag"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        security_obj,
                        "tag",
                        parsers,
                        "security_group.tag",
                    )
                elif h_item and h_item[security_obj].get("tag"):
                    h_item[security_obj] = {
                        "tag": h_item[security_obj].get("tag"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
        self.check_for_have_and_overidden(have)

    def _service_object_compare(self, want, have):
        service_obj = "service_object"
        services_obj = "services_object"
        port_obj = "port_object"
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type":
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        service_obj,
                        ["protocol"],
                    )
                else:
                    protocol = entry.get("protocol")
                    if protocol:
                        entry["name"] = "{0} {1}".format(name, protocol)
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if entry.get(service_obj):
                    if entry[service_obj].get("protocol"):
                        self._compare_object_diff(
                            entry,
                            h_item,
                            service_obj,
                            "protocol",
                            ["service_object"],
                            service_obj,
                        )
                elif entry.get(services_obj):
                    if h_item:
                        h_item = self.convert_list_to_dict(
                            val=h_item,
                            source="source_port",
                            destination="destination_port",
                        )
                    entry = self.convert_list_to_dict(
                        val=entry,
                        source="source_port",
                        destination="destination_port",
                    )
                    command_len = len(self.commands)
                    for k, v in iteritems(entry):
                        if h_item:
                            h_service_item = h_item.pop(k, {})
                            if h_service_item != v:
                                self.compare(
                                    [services_obj],
                                    want={services_obj: v},
                                    have={services_obj: h_service_item},
                                )
                        else:
                            temp_want = {"name": name, services_obj: v}
                            self.addcmd(temp_want, "og_name", True)

                            self.compare(
                                [services_obj],
                                want=temp_want,
                                have={},
                            )
                    if h_item and self.state in ["overridden", "replaced"]:
                        for k, v in iteritems(h_item):
                            temp_have = {"name": name, services_obj: v}
                            self.compare(
                                [services_obj],
                                want={},
                                have=temp_have,
                            )
                    if command_len < len(self.commands):
                        cmd = "object-group service {0}".format(name)
                        if cmd not in self.commands:
                            self.commands.insert(command_len, cmd)
                elif entry.get(port_obj):
                    protocol = entry.get("protocol")
                    if h_item:
                        h_item = self.convert_list_to_dict(
                            val=h_item,
                            source="source_port",
                            destination="destination_port",
                        )
                    entry = self.convert_list_to_dict(
                        val=entry,
                        source="source_port",
                        destination="destination_port",
                    )
                    command_len = len(self.commands)
                    for k, v in iteritems(entry):
                        h_port_item = h_item.pop(k, {})
                        if "http" in k and "_" in k:
                            # This condition is to TC of device behaviour, where if user tries to
                            # configure http it gets converted to www.
                            temp = k.split("_")[0]
                            h_port_item = {temp: "http"}
                        if h_port_item != v:
                            self.compare(
                                [port_obj],
                                want={port_obj: v},
                                have={port_obj: h_port_item},
                            )
                        elif not h_port_item:
                            temp_want = {"name": name, port_obj: v}
                            self.compare([port_obj], want=temp_want, have={})
                    if h_item and self.state in ["overridden", "replaced"]:
                        for k, v in iteritems(h_item):
                            temp_have = {"name": name, port_obj: v}
                            self.compare([port_obj], want={}, have=temp_have)
        self.check_for_have_and_overidden(have)

    def convert_list_to_dict(self, *args, **kwargs):
        temp = {}
        if kwargs["val"].get("services_object"):
            for every in kwargs["val"]["services_object"]:
                temp_key = every["protocol"]
                if "source_port" in every:
                    if "range" in every["source_port"]:
                        temp_key = (
                            "range"
                            + "_"
                            + str(every["source_port"]["range"]["start"])
                            + "_"
                            + str(every["source_port"]["range"]["end"])
                        )
                    else:
                        source_key = list(every["source_port"])[0]
                        temp_key = (
                            temp_key + "_" + source_key + "_" + every["source_port"][source_key]
                        )
                if "destination_port" in every:
                    if "range" in every["destination_port"]:
                        temp_key = (
                            "range"
                            + "_"
                            + str(every["destination_port"]["range"]["start"])
                            + "_"
                            + str(every["destination_port"]["range"]["end"])
                        )
                    else:
                        destination_key = list(every["destination_port"])[0]
                        temp_key = (
                            temp_key
                            + "_"
                            + destination_key
                            + "_"
                            + every["destination_port"][destination_key]
                        )
                temp.update({temp_key: every})
            return temp
        elif kwargs["val"].get("port_object"):
            for every in kwargs["val"]["port_object"]:
                if "range" in every:
                    temp_key = (
                        "start"
                        + "_"
                        + every["range"]["start"]
                        + "_"
                        + "end"
                        + "_"
                        + every["range"]["end"]
                    )
                else:
                    every_key = list(every)[0]
                    temp_key = every_key + "_" + every[every_key]
                temp.update({temp_key: every})
            return temp

    def _user_object_compare(self, want, have):
        user_obj = "user_object"
        parsers = ["user_object.user", "user_object.user_gp"]
        add_obj_cmd = False
        for name, entry in iteritems(want):
            h_item = have.pop(name, {})
            if entry != h_item and name != "object_type":
                if h_item and entry.get("group_object"):
                    self.addcmd(entry, "og_name", False)
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if h_item:
                    self._add_object_cmd(
                        entry,
                        h_item,
                        user_obj,
                        ["user", "user_group"],
                    )
                else:
                    add_obj_cmd = True
                    self.addcmd(entry, "og_name", False)
                    self.compare(["description"], entry, h_item)
                if entry.get("group_object"):
                    self._add_group_object_cmd(entry, h_item)
                    continue
                if entry[user_obj].get("user"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        user_obj,
                        "user",
                        ["user_object.user"],
                        "user_object.user",
                    )
                elif h_item and h_item[user_obj].get("user"):
                    h_item[user_obj] = {"user": h_item[user_obj].get("user")}
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
                if entry[user_obj].get("user_group"):
                    self._compare_object_diff(
                        entry,
                        h_item,
                        user_obj,
                        "user_group",
                        ["user_object.user_group"],
                        "user_object.user_gp",
                    )
                elif h_item and h_item[user_obj].get("user_group"):
                    h_item[user_obj] = {
                        "user_group": h_item[user_obj].get("user_group"),
                    }
                    if not add_obj_cmd:
                        self.addcmd(entry, "og_name", False)
                    self.compare(parsers, {}, h_item)
        self.check_for_have_and_overidden(have)

    def _add_object_cmd(self, want, have, object, object_elements):
        obj_cmd_added = False
        for each in object_elements:
            want_element = want[object].get(each) if want.get(object) else want
            have_element = have[object].get(each) if have.get(object) else have
            if (
                want_element
                and isinstance(want_element, list)
                and isinstance(want_element[0], dict)
            ):
                if want_element and have_element and want_element != have_element:
                    if not obj_cmd_added:
                        self.addcmd(want, "og_name", False)
                        self.compare(["description"], want, have)
                        obj_cmd_added = True
            else:
                if want_element and have_element and set(want_element) != set(have_element):
                    if not obj_cmd_added:
                        self.addcmd(want, "og_name", False)
                        self.compare(["description"], want, have)
                        obj_cmd_added = True

    def _add_group_object_cmd(self, want, have):
        if have and have.get("group_object"):
            want["group_object"] = list(
                set(want.get("group_object")) - set(have.get("group_object")),
            )
            have["group_object"] = list(
                set(have.get("group_object")) - set(want.get("group_object")),
            )
        for each in want["group_object"]:
            self.compare(["group_object"], {"group_object": each}, dict())
        if (
            (self.state == "replaced" or self.state == "overridden")
            and have
            and have.get("group_object")
        ):
            for each in have["group_object"]:
                self.compare(["group_object"], dict(), {"group_object": each})

    def _compare_object_diff(
        self,
        want,
        have,
        object,
        object_type,
        parsers,
        val,
    ):
        temp_have = copy.copy(have)
        temp_want = copy.copy(want)
        if temp_have and temp_have.get(object) and temp_have[object].get(object_type):
            want_diff = self.get_list_diff(
                temp_want,
                temp_have,
                object,
                object_type,
            )
            have_diff = [
                each
                for each in temp_have[object][object_type]
                if each not in temp_want[object][object_type]
            ]
            if have_diff:
                temp_have[object].pop(object_type)
        else:
            have_diff = []
            want_diff = temp_want[object].get(object_type)
        temp_want[object][object_type] = want_diff
        if have_diff or temp_have.get(object) and self.state in ("overridden", "replaced"):
            if have_diff:
                temp_have[object] = {object_type: have_diff}
                self.compare(parsers, {}, temp_have)
        self.addcmd(temp_want, val, False)



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang <haiyzhan@micosoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_openshiftmanagedclusterkubeconfig_info
version_added: '1.17.0'
short_description: Get admin kubeconfig of Azure Red Hat OpenShift Managed Cluster
description:
    - get kubeconfig of Azure Red Hat OpenShift Managed Cluster instance.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    name:
        description:
            - Resource name.
        required: true
        type: str
    path:
        description:
            - Destination filepath of kubeconfig file
        required: false
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Maxim Babushkin (@maxbab)
'''

EXAMPLES = '''
- name: Obtain kubeconfig file of ARO cluster
  azure_rm_openshiftmanagedclusterkubeconfig_info:
    name: myCluster
    resource_group: myResourceGroup
  register: kubeconf

- name: Print registered kubeconfig file
  debug:
    msg: "{{ kubeconf['kubeconfig'] }}"

- name: Fetch kubeconfig and save it as mycluster_kubeconfig filename
  azure_rm_openshiftmanagedclusterkubeconfig_info:
    name: myCluster
    resource_group: myResourceGroup
    path: ./files/mycluster_kubeconfig

- name: Fetch kubeconfig and save it to specified directory (file will be named as kubeconfig by default)
  azure_rm_openshiftmanagedclusterkubeconfig_info:
    name: myCluster
    resource_group: myResourceGroup
    path: ./files/
'''

RETURN = '''
kubeconfig:
    description:
        - kubeconfig value
    returned: always
    type: str
'''

import base64
import filecmp
import json
import os
import tempfile
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMOpenShiftManagedClustersKubeconfigInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str', required=True
            ),
            name=dict(
                type='str', required=True
            ),
            path=dict(
                type='str', required=False
            )
        )

        self.resource_group = None
        self.name = None
        self.path = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2021-09-01-preview'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        self.mgmt_client = None
        super(AzureRMOpenShiftManagedClustersKubeconfigInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)
        self.results = self.get_kubeconfig()
        if self.path and self.path_is_valid():
            self.write_kubeconfig_to_file()
        return self.results

    def get_kubeconfig(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.RedHatOpenShift' +
                    '/openShiftClusters' +
                    '/{{ cluster_name }}' +
                    '/listAdminCredentials')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ cluster_name }}', self.name)
        self.log("Fetch for kubeconfig from the cluster.")
        try:
            response = self.mgmt_client.query(self.url,
                                              'POST',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')
        return self.format_item(results)

    def format_item(self, item):
        d = {
            'kubeconfig': item.get('kubeconfig'),
        }
        return d

    def path_is_valid(self):
        if not os.path.basename(self.path):
            if os.path.isdir(self.path):
                self.log("Path is dir. Appending file name.")
                self.path += "kubeconfig"
            else:
                try:
                    self.log('Attempting to makedirs {0}'.format(self.path))
                    os.makedirs(self.path)
                except IOError as exc:
                    self.fail("Failed to create directory {0} - {1}".format(self.path, str(exc)))
                self.path += "kubeconfig"
        else:
            file_name = os.path.basename(self.path)
            path = self.path.replace(file_name, '')
            self.log('Checking path {0}'. format(path))
            # If the "path" is not defined, it's cwd.
            if path and not os.path.isdir(path):
                try:
                    self.log('Attempting to makedirs {0}'. format(path))
                    os.makedirs(path)
                except IOError as exc:
                    self.fail("Failed to create directory {0} - {1}".format(path, str(exc)))
        self.log("Validated path - {0}". format(self.path))
        return True

    def write_kubeconfig_to_file(self):
        decoded_bytes = base64.b64decode(self.results['kubeconfig'])
        decoded_string = decoded_bytes.decode("utf-8")

        if os.path.exists(self.path):
            self.log('Existing kubeconfig file found. Compare, to decide if needs to override')
            # If kubeconfig file already exists, compare it with the new file
            # If equal, do nothing, otherwise, override.
            tmp_kubeconfig = tempfile.TemporaryFile(mode='w')
            tmp_kubeconfig.write(decoded_string)
            tmp_kubeconfig.seek(0)

            # No need to close the temp file as it's closed by filecmp.cmp.
            if filecmp.cmp(tmp_kubeconfig.name, self.path):
                self.log("Files are identical. No need to override.")
                self.results['changed'] = False
                return

        self.log("Create {0} kubeconfig file.".format(self.path))
        try:
            with open(self.path, "w") as file:
                file.write(decoded_string)
        except Exception as exc:
            self.fail("Failed to write kubeconfig output to file - {0} to {1} - {2}".format(self.results['kubeconfig'],
                                                                                            self.path, exc))
        self.log("The {0} kubeconfig file has been created.")
        self.results['changed'] = True
        return


def main():
    AzureRMOpenShiftManagedClustersKubeconfigInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_localnetworkgateway

version_added: "2.4.0"

short_description: Manage Azure Local Network Gateway in a resource group

description:
    - Create, update or delete Azure Local Network Gateway in a resource group

options:
    resource_group:
        description:
            - The local network gateway's resource group.
        type: str
        required: true
    name:
        description:
            - The name of the local network gateway.
        type: str
        required: true
    location:
        description:
            - The location of the local network gateway.
        type: str
    local_network_address_space:
        description:
            - Local network site address space.
        type: dict
        suboptions:
            address_prefixes:
                description:
                    - A list of address blocks reserved for this virtual network in CIDR notation.
                type: list
                elements: str
    gateway_ip_address:
        description:
            - IP address of local network gateway.
        type: str
    fqdn:
        description:
            - FQDN of local network gateway.
        type: str
    bgp_settings:
        description:
            - Local network gateway's BGP speaker settings.
        type: dict
        suboptions:
            asn:
                description:
                    - The BGP speaker's ASN.
                type: int
            bgp_peering_address:
                description:
                    - The BGP peering address and BGP identifier of this BGP speaker.
                type: str
            peer_weight:
                description:
                    - The weight added to routes learned from this BGP speaker.
                type: int
    state:
        description:
            - Use C(present) to create or update a local network gateway.
            - Use C(absent) to delete the local network gateway.
        type: str
        default: present
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new local network gateway
  azure_rm_localnetworkgateway:
    resource_group: "{{ resource_group }}"
    name: "localgateway-name"
    local_network_address_space:
      address_prefixes:
        - 10.0.0.0/24
        - 20.0.0.0/24
    fqdn: fredtest.com
    tags:
      key: value
    bgp_settings:
      asn: 8
      bgp_peering_address: 10.3.0.1
      peer_weight: 3

- name: Delete local network gateway
  azure_rm_localnetworkgateway:
    resource_group: "{{ resource_group }}"
    name: "localgateway-name"
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the Azure Local Network Gateway resource.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxx-xxxx/resourceGroups/testRG/providers/Microsoft.Network/localNetworkGateways/testgateway"
        bgp_settings:
            description:
                - Local network gateway's BGP speaker settings.
            type: complex
            contains:
                asn:
                    description:
                        - The BGP speaker's ASN.
                    type: int
                    returned: always
                    sample: 10
                bgp_peering_address:
                    description:
                        - The BGP peering address and BGP identifier of this BGP speaker.
                    type: str
                    returned: always
                    sample: 10.0.0.3
                peer_weight:
                    description:
                        - The weight added to routes learned from this BGP speaker.
                    type: int
                    returned: always
                    sample: 0
        fqdn:
            description:
                - FQDN of local network gateway.
            type: str
            returned: always
            sample: testfqdn.com
        gateway_ip_address:
            description:
                - IP address of local network gateway.
            type: str
            returned: always
            sample: 10.1.1.1
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: b5a32693-2e75-49e0-9137-ded19db658d6
        local_network_address_space:
            description:
                - Local network site address space.
            type: complex
            contains:
                address_prefixes:
                    description:
                        - A list of address blocks reserved for this virtual network in CIDR notation.
                    type: list
                    returned: always
                    sample: ["10.0.0.0/24", "20.0.0.0/24"]
        location:
            description:
                - The resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: testgateway
        provisioning_state:
            description:
                - The provisioning state of the local network gateway resource.
            type: str
            returned: always
            sample: Succeeded
        tags:
            description:
                - The resource tags.
            type: str
            returned: always
            sample: {'key1': 'value1', 'key2': 'value2'}
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/localNetworkGateways
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
    from azure.core.polling import LROPoller
except Exception:
    # handled in azure_rm_common
    pass


bgp_settings_spec = dict(
    asn=dict(type='int'),
    bgp_peering_address=dict(type='str'),
    peer_weight=dict(type='int'),
)


local_network_address_space_spec = dict(
    address_prefixes=dict(type='list', elements='str')
)


class AzureRMNetworkGateWay(AzureRMModuleBase):
    """Utility class to get Azure Kubernetes Service Credentials facts"""

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            location=dict(type='str'),
            local_network_address_space=dict(type='dict', options=local_network_address_space_spec),
            gateway_ip_address=dict(type='str'),
            fqdn=dict(type='str'),
            bgp_settings=dict(type='dict', options=bgp_settings_spec),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.name = None
        self.location = None
        self.local_network_address_space = None
        self.gateway_ip_address = None
        self.fqdn = None
        self.tags = None
        self.bgp_settings = None

        self.results = dict(
            changed=False,
            state=[],
        )
        mutually_exclusive = [['gateway_ip_address', 'fqdn']]

        super(AzureRMNetworkGateWay, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    mutually_exclusive=mutually_exclusive,
                                                    supports_tags=True,
                                                    facts_module=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec) + ['tags']:
            setattr(self, key, kwargs[key])

        if not self.location:
            # Set default location
            resource_group = self.get_resource_group(self.resource_group)
            self.location = resource_group.location

        old_response = self.get_local_network_gateway()
        changed = False
        update_tags = False

        response = None
        if self.state == 'present':
            if old_response is not None:
                if self.fqdn is not None and self.fqdn != old_response['fqdn']:
                    changed = True
                else:
                    self.fqdn = old_response['fqdn']
                if self.gateway_ip_address is not None and self.gateway_ip_address != old_response['gateway_ip_address']:
                    changed = True
                else:
                    self.gateway_ip_address = old_response['gateway_ip_address']
                if self.bgp_settings is not None and\
                   not all(self.bgp_settings.get(key) == old_response['bgp_settings'].get(key) for key in self.bgp_settings.keys()):
                    changed = True
                if self.local_network_address_space is not None:
                    if old_response['local_network_address_space'].get('address_prefixes') is not None:
                        new_address = list(set(self.local_network_address_space['address_prefixes'] +
                                           old_response['local_network_address_space']['address_prefixes']))
                        if len(new_address) > len(old_response['local_network_address_space'].get('address_prefixes')):
                            changed = True
                        self.local_network_address_space['address_prefixes'] = new_address
                    else:
                        changed = True
                else:
                    self.local_network_address_space['address_prefixes'] = old_response['local_network_address_space'].get('address_prefixes')

                update_tags, new_tags = self.update_tags(old_response.get('tags'))
                if update_tags:
                    # response = self.update_local_network_gateway_tags(new_tags)
                    self.fail("Can't update the local network gateway tags, Exception code as AllPropertiesAreReadOnly")
                    changed = True
            else:
                changed = True

            local_network_address_space = None
            if self.local_network_address_space is not None:
                local_network_address_space = self.network_models.AddressSpace(address_prefixes=self.local_network_address_space['address_prefixes'])
            bgp_settings = None
            if self.bgp_settings is not None:
                bgp_settings = self.network_models.BgpSettings(asn=self.bgp_settings.get('asn'),
                                                               bgp_peering_address=self.bgp_settings.get('bgp_peering_address'),
                                                               peer_weight=self.bgp_settings.get('peer_weight'))

            gateway_resource = self.network_models.LocalNetworkGateway(location=self.location,
                                                                       tags=self.tags,
                                                                       gateway_ip_address=self.gateway_ip_address,
                                                                       fqdn=self.fqdn,
                                                                       local_network_address_space=local_network_address_space,
                                                                       bgp_settings=bgp_settings)
            if changed:
                if not self.check_mode:
                    response = self.create_or_update_local_network_gateway(gateway_resource)

            if old_response is not None:
                update_tags, new_tags = self.update_tags(old_response.get('tags'))
                if update_tags:
                    if not self.check_mode:
                        response = self.update_local_network_gateway_tags(new_tags)
                    changed = True
        else:
            if not self.check_mode:
                if old_response is not None:
                    self.delete_local_network_gateway()
                    changed = True
                    response = None
            else:
                changed = True

        if response is None:
            response = old_response
        self.results['state'] = response
        self.results['changed'] = changed
        return self.results

    def get_local_network_gateway(self):
        """Gets the specified local network gateway in a resource group"""
        response = None
        try:
            response = self.network_client.local_network_gateways.get(self.resource_group, self.name)
        except HttpResponseError as ec:
            self.log("Gets the specified local network gateway in a resource group Failed, Exception as {0}".format(ec))
            return None
        return self.format_response(response)

    def create_or_update_local_network_gateway(self, body):
        """Create or Update local network gateway"""
        response = None
        try:
            response = self.network_client.local_network_gateways.begin_create_or_update(self.resource_group, self.name, body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except HttpResponseError as ec:
            self.fail("Create or Updated a local network gateway in a resource group Failed, Exception as {0}".format(ec))

        return self.format_response(response)

    def update_local_network_gateway_tags(self, tags):
        """Updates a local network gateway tags"""
        response = None
        try:
            response = self.network_client.local_network_gateways.update_tags(self.resource_group, self.name, tags)
        except HttpResponseError as ec:
            self.fail("Update a local network gateway tags Failed, Exception as {0}".format(ec))
        return self.format_response(response)

    def delete_local_network_gateway(self):
        """Deletes the specified local network gateway"""
        try:
            self.network_client.local_network_gateways.begin_delete(self.resource_group, self.name)
        except HttpResponseError as ec:
            self.fail("Deletes the specified local network gateway Failed, Exception as {0}".format(ec))
        return None

    def format_response(self, item):
        result = dict(
            id=item.id,
            name=item.name,
            location=item.location,
            type=item.type,
            tags=item.tags,
            etag=item.etag,
            local_network_address_space=dict(),
            gateway_ip_address=item.gateway_ip_address,
            fqdn=item.fqdn,
            provisioning_state=item.provisioning_state,
            bgp_settings=dict(),
        )

        if item.local_network_address_space is not None:
            result['local_network_address_space']['address_prefixes'] = item.local_network_address_space.address_prefixes
        if item.bgp_settings is not None:
            result['bgp_settings']['asn'] = item.bgp_settings.asn
            result['bgp_settings']['bgp_peering_address'] = item.bgp_settings.bgp_peering_address
            result['bgp_settings']['peer_weight'] = item.bgp_settings.peer_weight
        return result


def main():
    """Main module execution code path"""

    AzureRMNetworkGateWay()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2017 Willem van Ketwich
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Author:
#   - Willem van Ketwich <willem@vanketwich.com.au>
#
# Common functionality to be used by the modules:
#   - cloudfront_distribution
#   - cloudfront_invalidation
#   - cloudfront_origin_access_identity

"""
Common cloudfront facts shared between modules
"""

from functools import partial

try:
    import botocore
except ImportError:
    pass

from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from .retries import AWSRetry
from .tagging import boto3_tag_list_to_ansible_dict


class CloudFrontFactsServiceManagerFailure(Exception):
    pass


def cloudfront_facts_keyed_list_helper(list_to_key):
    result = dict()
    for item in list_to_key:
        distribution_id = item["Id"]
        if "Items" in item["Aliases"]:
            result.update({alias: item for alias in item["Aliases"]["Items"]})
        result.update({distribution_id: item})
    return result


@AWSRetry.jittered_backoff()
def _cloudfront_paginate_build_full_result(client, client_method, **kwargs):
    paginator = client.get_paginator(client_method)
    return paginator.paginate(**kwargs).build_full_result()


class CloudFrontFactsServiceManager:
    """Handles CloudFront Facts Services"""

    CLOUDFRONT_CLIENT_API_MAPPING = {
        "get_distribution": {
            "error": "Error describing distribution",
        },
        "get_distribution_config": {
            "error": "Error describing distribution configuration",
        },
        "get_origin_access_identity": {
            "error": "Error describing origin access identity",
            "client_api": "get_cloud_front_origin_access_identity",
        },
        "get_origin_access_identity_config": {
            "error": "Error describing origin access identity configuration",
            "client_api": "get_cloud_front_origin_access_identity_config",
        },
        "get_streaming_distribution": {
            "error": "Error describing streaming distribution",
        },
        "get_streaming_distribution_config": {
            "error": "Error describing streaming distribution",
        },
        "get_invalidation": {
            "error": "Error describing invalidation",
        },
        "list_distributions_by_web_acl_id": {
            "error": "Error listing distributions by web acl id",
            "post_process": lambda x: cloudfront_facts_keyed_list_helper(
                x.get("DistributionList", {}).get("Items", [])
            ),
        },
    }

    CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING = {
        "list_origin_access_identities": {
            "error": "Error listing cloud front origin access identities",
            "client_api": "list_cloud_front_origin_access_identities",
            "key": "CloudFrontOriginAccessIdentityList",
        },
        "list_distributions": {
            "error": "Error listing distributions",
            "key": "DistributionList",
            "keyed": True,
        },
        "list_invalidations": {"error": "Error listing invalidations", "key": "InvalidationList"},
        "list_streaming_distributions": {
            "error": "Error listing streaming distributions",
            "key": "StreamingDistributionList",
            "keyed": True,
        },
    }

    def __init__(self, module):
        self.module = module
        self.client = module.client("cloudfront", retry_decorator=AWSRetry.jittered_backoff())

    def describe_cloudfront_property(self, client_method, error, post_process, **kwargs):
        fail_if_error = kwargs.pop("fail_if_error", True)
        try:
            method = getattr(self.client, client_method)
            api_kwargs = snake_dict_to_camel_dict(kwargs, capitalize_first=True)
            result = method(aws_retry=True, **api_kwargs)
            result.pop("ResponseMetadata", None)
            if post_process:
                result = post_process(result)
            return result
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            if not fail_if_error:
                raise
            self.module.fail_json_aws(e, msg=error)

    def paginate_list_cloudfront_property(self, client_method, key, default_keyed, error, **kwargs):
        fail_if_error = kwargs.pop("fail_if_error", True)
        try:
            keyed = kwargs.pop("keyed", default_keyed)
            api_kwargs = snake_dict_to_camel_dict(kwargs, capitalize_first=True)
            result = _cloudfront_paginate_build_full_result(self.client, client_method, **api_kwargs)
            items = result.get(key, {}).get("Items", [])
            if keyed:
                items = cloudfront_facts_keyed_list_helper(items)
            return items
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            if not fail_if_error:
                raise
            self.module.fail_json_aws(e, msg=error)

    def __getattr__(self, name):
        if name in self.CLOUDFRONT_CLIENT_API_MAPPING:
            client_method = self.CLOUDFRONT_CLIENT_API_MAPPING[name].get("client_api", name)
            error = self.CLOUDFRONT_CLIENT_API_MAPPING[name].get("error", "")
            post_process = self.CLOUDFRONT_CLIENT_API_MAPPING[name].get("post_process")
            return partial(self.describe_cloudfront_property, client_method, error, post_process)

        elif name in self.CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING:
            client_method = self.CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING[name].get("client_api", name)
            error = self.CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING[name].get("error", "")
            key = self.CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING[name].get("key")
            keyed = self.CLOUDFRONT_CLIENT_PAGINATE_API_MAPPING[name].get("keyed", False)
            return partial(self.paginate_list_cloudfront_property, client_method, key, keyed, error)

        raise CloudFrontFactsServiceManagerFailure(f"Method {name} is not currently supported")

    def summary(self):
        summary_dict = {}
        summary_dict.update(self.summary_get_distribution_list(False))
        summary_dict.update(self.summary_get_distribution_list(True))
        summary_dict.update(self.summary_get_origin_access_identity_list())
        return summary_dict

    def summary_get_origin_access_identity_list(self):
        try:
            origin_access_identities = []
            for origin_access_identity in self.list_origin_access_identities():
                oai_id = origin_access_identity["Id"]
                oai_full_response = self.get_origin_access_identity(oai_id)
                oai_summary = {"Id": oai_id, "ETag": oai_full_response["ETag"]}
                origin_access_identities.append(oai_summary)
            return {"origin_access_identities": origin_access_identities}
        except botocore.exceptions.ClientError as e:
            self.module.fail_json_aws(e, msg="Error generating summary of origin access identities")

    def list_resource_tags(self, resource_arn):
        return self.client.list_tags_for_resource(Resource=resource_arn, aws_retry=True)

    def summary_get_distribution_list(self, streaming=False):
        try:
            list_name = "streaming_distributions" if streaming else "distributions"
            key_list = ["Id", "ARN", "Status", "LastModifiedTime", "DomainName", "Comment", "PriceClass", "Enabled"]
            distribution_list = {list_name: []}
            distributions = (
                self.list_streaming_distributions(keyed=False) if streaming else self.list_distributions(keyed=False)
            )
            for dist in distributions:
                temp_distribution = {k: dist[k] for k in key_list}
                temp_distribution["Aliases"] = list(dist["Aliases"].get("Items", []))
                temp_distribution["ETag"] = self.get_etag_from_distribution_id(dist["Id"], streaming)
                if not streaming:
                    temp_distribution["WebACLId"] = dist["WebACLId"]
                    invalidation_ids = self.get_list_of_invalidation_ids_from_distribution_id(dist["Id"])
                    if invalidation_ids:
                        temp_distribution["Invalidations"] = invalidation_ids
                resource_tags = self.list_resource_tags(dist["ARN"])
                temp_distribution["Tags"] = boto3_tag_list_to_ansible_dict(resource_tags["Tags"].get("Items", []))
                distribution_list[list_name].append(temp_distribution)
            return distribution_list
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error generating summary of distributions")

    def get_etag_from_distribution_id(self, distribution_id, streaming):
        distribution = {}
        if not streaming:
            distribution = self.get_distribution(id=distribution_id)
        else:
            distribution = self.get_streaming_distribution(id=distribution_id)
        return distribution["ETag"]

    def get_list_of_invalidation_ids_from_distribution_id(self, distribution_id):
        try:
            return list(map(lambda x: x["Id"], self.list_invalidations(distribution_id=distribution_id)))
        except botocore.exceptions.ClientError as e:
            self.module.fail_json_aws(e, msg="Error getting list of invalidation ids")

    def get_distribution_id_from_domain_name(self, domain_name):
        try:
            distribution_id = ""
            distributions = self.list_distributions(keyed=False)
            distributions += self.list_streaming_distributions(keyed=False)
            for dist in distributions:
                if any(str(alias).lower() == domain_name.lower() for alias in dist["Aliases"].get("Items", [])):
                    distribution_id = dist["Id"]
            return distribution_id
        except botocore.exceptions.ClientError as e:
            self.module.fail_json_aws(e, msg="Error getting distribution id from domain name")

    def get_aliases_from_distribution_id(self, distribution_id):
        try:
            distribution = self.get_distribution(id=distribution_id)
            return distribution["Distribution"]["DistributionConfig"]["Aliases"].get("Items", [])
        except botocore.exceptions.ClientError as e:
            self.module.fail_json_aws(e, msg="Error getting list of aliases from distribution_id")



#!/usr/bin/python
#
# Copyright (c) 2020 Aparna Patil(@aparna-patil)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_privatednsrecordset_info

version_added: "1.1.0"

short_description: Get Private DNS Record Set facts

description:
    - Get facts for a specific DNS Record Set in a Private DNS Zone, or a specific type of DNS record in all zones or
      one zone etc.

options:
    relative_name:
        description:
            - Only show results for a Record Set.
        type: str
    resource_group:
        description:
            - Limit results by resource group. Required when filtering by name or type.
        type: str
    zone_name:
        description:
            - Limit results by zones. Required when filtering by name or type.
        type: str
    record_type:
        description:
            - Limit record sets by record type.
        type: str
    top:
        description:
            - Limit the maximum number of record sets to return.
        type: int

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Aparna Patil (@aparna-patil)

'''

EXAMPLES = '''
- name: Get facts for one record set in one Private DNS Zone
  azure_rm_privatednsrecordset_info:
    resource_group: myResourceGroup
    zone_name: newzone.com
    relative_name: servera
    record_type: A
- name: Get facts for all Type A record sets in a Private DNS Zone
  azure_rm_privatednsrecordset_info:
    resource_group: myResourceGroup
    zone_name: newzone.com
    record_type: A
- name: Get all record sets in a Private DNS Zone
  azure_rm_privatednsrecordset_info:
    resource_group: myResourceGroup
    zone_name: newzone.com
'''

RETURN = '''
dnsrecordsets:
    description:
        - Gets a list of recordsets dict in a Private DNS zone.
    returned: always
    type: list
    elements: dict
    sample: [
        {
            "fqdn": "servera.newzone.com.",
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/
                   Microsoft.Network/privateDnsZones/newzone.com/A/servera",
            "record_type": "A",
            "records": [
                {
                    "ipv4_address": "10.10.10.10"
                }
            ],
            "relative_name": "servera",
            "time_to_live": 3600
        },
        {
            "fqdn": "serverb.newzone.com.",
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/
                   Microsoft.Network/privateDnsZones/newzone.com/A/serverb",
            "record_type": "A",
            "records": [
                {
                    "ipv4_address": "10.10.10.11"
                }
            ],
            "relative_name": "serverb",
            "time_to_live": 3600
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'RecordSet'


RECORDSET_VALUE_MAP = dict(
    A='a_records',
    AAAA='aaaa_records',
    CNAME='cname_record',
    MX='mx_records',
    PTR='ptr_records',
    SRV='srv_records',
    TXT='txt_records',
    SOA='soa_record'
)


class AzureRMPrivateDNSRecordSetInfo(AzureRMModuleBase):

    def __init__(self):

        # define user inputs into argument
        self.module_arg_spec = dict(
            relative_name=dict(type='str'),
            resource_group=dict(type='str'),
            zone_name=dict(type='str'),
            record_type=dict(type='str'),
            top=dict(type='int')
        )

        # store the results of the module operation
        self.results = dict(
            changed=False,
        )

        self.relative_name = None
        self.resource_group = None
        self.zone_name = None
        self.record_type = None
        self.top = None

        super(AzureRMPrivateDNSRecordSetInfo, self).__init__(self.module_arg_spec, supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if not self.top or self.top <= 0:
            self.top = None

        # create conditionals to catch errors when calling record facts
        if self.relative_name and not self.resource_group:
            self.fail("Parameter error: resource group required when filtering by name or record type.")
        if self.relative_name and not self.zone_name:
            self.fail("Parameter error: DNS Zone required when filtering by name or record type.")

        results = []
        # list the conditions for what to return based on input
        if self.relative_name is not None:
            # if there is a name listed, they want only facts about that specific Record Set itself
            results = self.get_item()
        elif self.record_type:
            # else, they just want all the record sets of a specific type
            results = self.list_type()
        elif self.zone_name:
            # if there is a zone name listed, then they want all the record sets in a zone
            results = self.list_zone()

        self.results['dnsrecordsets'] = self.curated_list(results)
        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.relative_name))
        item = None
        results = []

        # try to get information for specific Record Set
        try:
            item = self.private_dns_client.record_sets.get(self.resource_group,
                                                           self.zone_name,
                                                           self.record_type,
                                                           self.relative_name)
        except ResourceNotFoundError:
            pass

        results = [item]
        return results

    def list_type(self):
        self.log('Lists the record sets of a specified type in a Private DNS zone')
        try:
            response = self.private_dns_client.record_sets.list_by_type(self.resource_group,
                                                                        self.zone_name,
                                                                        self.record_type,
                                                                        top=self.top)
        except Exception as exc:
            self.fail("Failed to list for record type {0} - {1}".format(self.record_type, str(exc)))

        results = []
        for item in response:
            results.append(item)
        return results

    def list_zone(self):
        self.log('Lists all record sets in a Private DNS zone')
        try:
            response = self.private_dns_client.record_sets.list(self.resource_group, self.zone_name, top=self.top)
        except Exception as exc:
            self.fail("Failed to list for zone {0} - {1}".format(self.zone_name, str(exc)))

        results = []
        for item in response:
            results.append(item)
        return results

    def curated_list(self, raws):
        return [self.record_to_dict(item) for item in raws] if raws else []

    def record_to_dict(self, record):
        record_type = record.type[len('Microsoft.Network/privateDnsZones/'):]
        records = getattr(record, RECORDSET_VALUE_MAP.get(record_type))
        if records:
            if not isinstance(records, list):
                records = [records]
        else:
            records = []
        return dict(
            id=record.id,
            relative_name=record.name,
            record_type=record_type,
            records=[x.as_dict() for x in records],
            time_to_live=record.ttl,
            fqdn=record.fqdn
        )


def main():
    AzureRMPrivateDNSRecordSetInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_network_profiles_for_sites_v1_info
short_description: Information module for Network Devices Network Profiles For Sites V1
description:
- Get all Network Devices Network Profiles For Sites V1.
- Get Network Devices Network Profiles For Sites V1 by id.
- Retrieves a network profile for sites by id.
- Retrieves the list of network profiles for sites.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter. The first record to show for this page; the first record is numbered 1.
    type: float
  limit:
    description:
    - Limit query parameter. The number of records to show for this page.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A property within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. Whether ascending or descending order should be used to sort the response.
    type: str
  type:
    description:
    - Type query parameter. Filter responses to only include profiles of a given type.
    type: str
  id:
    description:
    - Id path parameter. The 'id' of the network profile, retrievable from 'GET /intent/api/v1/networkProfilesForSites'.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design RetrieveANetworkProfileForSitesByIdV1
  description: Complete reference of the RetrieveANetworkProfileForSitesByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieve-a-network-profile-for-sites-by-id
- name: Cisco DNA Center documentation for Site Design RetrievesTheListOfNetworkProfilesForSitesV1
  description: Complete reference of the RetrievesTheListOfNetworkProfilesForSitesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-list-of-network-profiles-for-sites
notes:
  - SDK Method used are
    site_design.SiteDesign.retrieve_a_network_profile_for_sites_by_id_v1,
    site_design.SiteDesign.retrieves_the_list_of_network_profiles_for_sites_v1,

  - Paths used are
    get /dna/intent/api/v1/networkProfilesForSites,
    get /dna/intent/api/v1/networkProfilesForSites/{id},

"""

EXAMPLES = r"""
- name: Get all Network Devices Network Profiles For Sites V1
  cisco.dnac.network_devices_network_profiles_for_sites_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    sortBy: string
    order: string
    type: string
  register: result

- name: Get Network Devices Network Profiles For Sites V1 by id
  cisco.dnac.network_devices_network_profiles_for_sites_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "type": "string"
      }
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: team
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller team.
description:
    - Create, update, or destroy Automation Platform Controller teams. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - Name to use for the team.
      required: True
      type: str
    new_name:
      description:
        - To use when changing a team's name.
      type: str
    description:
      description:
        - The description to use for the team.
      type: str
    organization:
      description:
        - Organization name, ID, or named URL the team should be made a member of.
      required: True
      type: str
    state:
      description:
        - Desired state of the resource.
      choices: ["present", "absent", "exists"]
      default: "present"
      type: str
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Create team
  team:
    name: Team Name
    description: Team Description
    organization: test-org
    state: present
    controller_config_file: "~/tower_cli.cfg"
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        description=dict(),
        organization=dict(required=True),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get('new_name')
    description = module.params.get('description')
    organization = module.params.get('organization')
    state = module.params.get('state')

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    org_id = module.resolve_name_to_id('organizations', organization)

    # Attempt to look up team based on the provided name and org ID
    team = module.get_one('teams', name_or_id=name, check_exists=(state == 'exists'), **{'data': {'organization': org_id}})

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(team)

    # Create the data that gets sent for create and update
    team_fields = {'name': new_name if new_name else (module.get_item_name(team) if team else name), 'organization': org_id}
    if description is not None:
        team_fields['description'] = description

    # If the state was present and we can let the module build or update the existing team, this will return on its own
    module.create_or_update_if_needed(team, team_fields, endpoint='teams', item_type='team')


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_provision_device_v1_info
short_description: Information module for Sda Provision Device V1
description:
- Get all Sda Provision Device V1.
- Get Provisioned Wired Device.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceManagementIpAddress:
    description:
    - DeviceManagementIpAddress query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetProvisionedWiredDeviceV1
  description: Complete reference of the GetProvisionedWiredDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-provisioned-wired-device
notes:
  - SDK Method used are
    sda.Sda.get_provisioned_wired_device_v1,

  - Paths used are
    get /dna/intent/api/v1/business/sda/provision-device,

"""

EXAMPLES = r"""
- name: Get all Sda Provision Device V1
  cisco.dnac.sda_provision_device_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceManagementIpAddress: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "deviceManagementIpAddress": "string",
      "siteNameHierarchy": "string",
      "status": "string",
      "description": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2024, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: cli_restore
author: Sagar Paul (@KB-perByte)
short_description: Restore device configuration to network devices over network_cli
description:
- This module provides platform agnostic way of restore text based configuration to
  network devices over network_cli connection plugin.
- The module uses the platforms `config replace` commands to restore
  backup configuration that is already copied over to the appliance.
version_added: 6.1.0
extends_documentation_fragment:
- ansible.netcommon.network_agnostic
options:
  filename:
    description:
    - Filename of the backup file, present in the appliance where the restore operation
      is to be performed. Check appliance for the configuration backup file name.
    type: str
  path:
    description:
    - The location in the target appliance where the file containing the backup exists.
      The path and the filename together create the input to the config replace command,
    - For an IOSXE appliance the path pattern is flash://<filename>
    type: str
"""

EXAMPLES = """
- name: Restore IOS-XE configuration
  ansible.netcommon.cli_restore:
    filename: backupDday.cfg
    path: flash://

# Command fired
# -------------
# config replace flash://backupDday.cfg force

# Task Output
# -----------
#
# ok: [BATMON] => changed=false
#   __restore__: |-
#     The rollback configlet from the last pass is listed below:
#     ********
#     !List of Rollback Commands:
#     Building configuration...
#     Current configuration : 3781 bytes
#     end
#     ********
#
#
#     Rollback aborted after 5 passes
#     The following commands are failed to apply to the IOS image.
#     ********
#     Building configuration...
#     Current configuration : 3781 bytes
#     ********
#   invocation:
#     module_args:
#       filename: backupDday.cfg
"""

RETURN = """
"""

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection, ConnectionError


def validate_args(module, device_operations):
    """validate param if it is supported on the platform"""
    feature_list = []

    for feature in feature_list:
        if module.params[feature]:
            supports_feature = device_operations.get(f"supports_{feature}")
            if supports_feature is None:
                module.fail_json(
                    msg=f"This platform does not specify whether {feature} is supported or not. "
                    "Please report an issue against this platform's cliconf plugin."
                )
            elif not supports_feature:
                module.fail_json(msg=f"Option {feature} is not supported on this platform")


def main():
    """main entry point for execution"""
    argument_spec = dict(
        filename=dict(type="str"),
        path=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
    )

    result = {"changed": False}
    connection = Connection(module._socket_path)
    try:
        running = connection.restore(
            filename=module.params["filename"],
            path=module.params["path"],
        )
    except ConnectionError as exc:
        if exc.code == -32601:  # Method not found
            msg = "This platform is not supported with cli_restore. Please report an issue against this platform's cliconf plugin."
            module.fail_json(msg, code=exc.code)
        else:
            module.fail_json(msg=to_text(exc, errors="surrogate_then_replace").strip())
    result["__restore__"] = running
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: trustsec_vn_vlan_mapping_bulk_create
short_description: Resource module for Trustsec VN VLAN Mapping Bulk Create
description:
- Manage operation create of the resource Trustsec VN VLAN Mapping Bulk Create.
version_added: '2.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Trustsec VN VLAN Mapping Bulk Create's payload.
    elements: dict
    suboptions:
      id:
        description: Identifier of the VN-VLAN Mapping.
        type: str
      isData:
        description: Flag which indicates whether the VLAN is data or voice type.
        type: bool
      isDefaultVLAN:
        description: Flag which indicates if the VLAN is default.
        type: bool
      lastUpdate:
        description: Timestamp for the last update of the VN-VLAN Mapping.
        type: str
      maxValue:
        description: Max value.
        type: int
      name:
        description: Name of the VLAN.
        type: str
      vnId:
        description: Identifier for the associated Virtual Network which is required
          unless its name is provided.
        type: str
      vnName:
        description: Name of the associated Virtual Network to be used for identity
          if id is not provided.
        type: str
    type: list
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for vnVlanMapping
  description: Complete reference of the vnVlanMapping API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!trustsec-openapi
notes:
  - SDK Method used are
    vn_vlan_mapping.VnVlanMapping.bulk_create_vn_vlan_mappings,

  - Paths used are
    post /api/v1/trustsec/vnvlanmapping/bulk/create,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.trustsec_vn_vlan_mapping_bulk_create:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    payload:
    - id: string
      isData: true
      isDefaultVlan: true
      lastUpdate: string
      maxValue: 0
      name: string
      vnId: string
      vnName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string"
    }
"""
